{
Copyright (C) 2007  Andreas Frie

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
}
{
@abstract(@name implements a square and a round Status-LED)
@author(Andreas Frie <andi.at.andifriess.dot.com>)
@created(12.01.2007)
@cvs($Date: 2007-01-26 22:08:24 +0100 (Fr, 26 Jän 2007) $)
@cvs($Rev: 6 $)
@cvs($Author: af0815 $)
@cvs($URL: https://openbde.svn.sourceforge.net/svnroot/openbde/trunk/components/LED_status/led_status.pas $)
}
unit led_status;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LCLType, LCLProc, LMessages, Graphics,
  GraphType, GraphMath,  LCLIntf, ClipBrd, ActnList, Controls, LResources;

type

  TLEDStatusAlignment = (slLeftJustify, slRightJustify);

  TLEDStatusStyle = (slsQuad, slsRound);

  TLEDStatusState = (slGrayed, slNolight, slLight);

{ TLEDStatus }

TLEDStatus = class(TCustomControl)
private
  FAlignment: TLEDStatusAlignment;
  FColorLight: TColor;
  FColorNolight: TColor;
  FLEDStatusState: TLEDStatusState;
  FTextStyle: TTextStyle;
  TLEDStatusStyle: TLEDStatusStyle;
  FMouseInLEDStatus: Boolean;
  procedure DoAutoSize; override;
  procedure SetAlignment(const AValue: TLEDStatusAlignment);
  procedure SetLEDStatusState(const AValue: TLEDStatusState);
  procedure SetLEDStatusStyle(const AValue: TLEDStatusStyle);
  procedure SetTextStyle(const AValue: TTextStyle);

protected
  procedure PaintLED(recPaint: TRect); virtual; abstract;
  procedure PaintText(recPaint: TRect); virtual;
  procedure CMMouseEnter(var Message: TLMMouse); message CM_MOUSEENTER;
  procedure CMMouseLeave(var Message: TLMMouse); message CM_MOUSELEAVE;
  //procedure WMMouseUp(var Message: TLMMouseEvent); message LM_LBUTTONUP; //22.01.07-ud. Herausgenommen...

public
  procedure Click; override;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  procedure Paint; override;
  property MouseInLEDStatus: Boolean read FMouseInLEDStatus;
  property TextStyle : TTextStyle read FTextStyle write SetTextStyle;
published
  property Alignment: TLEDStatusAlignment read FAlignment write SetAlignment;
  property State: TLEDStatusState read FLEDStatusState write SetLEDStatusState;
  property Style: TLEDStatusStyle read TLEDStatusStyle write SetLEDStatusStyle;
  property ColorOn : TColor read FColorLight write FColorLight default clRed;
  property ColorOff : TColor read FColorNolight write FColorNolight default clGray;

  property Align;
  property AutoSize;
  property TabStop;

  property Anchors;
  property Constraints;
  property Hint;
  property Font;
  property OnClick;
  property OnDblClick;
  property OnEnter;
  property OnExit;
  property OnKeyDown;
  property OnKeyUp;
  property OnKeyPress;
  property OnMouseDown;
  property OnMouseUp;
  property OnMouseMove;
  property Visible;
  property Caption;
  property Enabled;
  property ShowHint;
  property ParentFont;
  property ParentShowHint;
  property TabOrder;
end;

{ TLEDStatusRound }

TLEDStatusRound = class(TLEDStatus)
private

protected

public
  procedure PaintLED(recPaint: TRect); override;
published

end;

{ TLEDStatusSquare }

TLEDStatusSquare = class(TLEDStatus)
private

protected

public
  procedure PaintLED(recPaint: TRect); override;
published

end;

procedure Register;

implementation


{ TStatusLED private}
Procedure TLEDStatus.DoAutoSize;
var
  aDC : hDC;
  aRect : TRect;
begin
  // check if something to do
  If AutoSizing or not AutoSize then
    Exit;
  if (not HandleAllocated) or ([csLoading,csDestroying]*ComponentState<>[]) then
    Exit;
  // ok, we must do something
  // set a flag to prohibit recursing !
  AutoSizing := True;
  // create a context -  dont forget to release !!
  aDC := GetDC(Handle);
  Try
    // get the region of working
    aRect := Rect(0,0, Width, Height);
    DrawText(aDC, PChar(Caption), Length(Caption), aRect,
      DT_CalcRect or DT_NOPrefix);
    // adjust the minimum size
    If aRect.Right > Width then
      Width := aRect.Right + 25;
    If aRect.Bottom > Height then
      Height := aRect.Bottom + 2;
  Finally
    // release context !!
    ReleaseDC(Handle, aDC);
    AutoSizing := False;
  end;
end;

procedure TLEDStatus.SetAlignment(const AValue: TLEDStatusAlignment);
begin
  if FAlignment=AValue then exit;
  FAlignment:=AValue;
end;

procedure TLEDStatus.SetLEDStatusState(const AValue: TLEDStatusState);
begin
  if FLEDStatusState=AValue then exit;
  FLEDStatusState:=AValue;
end;

procedure TLEDStatus.SetLEDStatusStyle(const AValue: TLEDStatusStyle);
begin
  if TLEDStatusStyle=AValue then exit;
  TLEDStatusStyle:=AValue;
end;

procedure TLEDStatus.SetTextStyle(const AValue: TTextStyle);
begin
  FTextStyle := AValue;
end;

procedure TLEDStatus.PaintText(recPaint: TRect);
begin
  // draw the text for the component if needed
  Canvas.TextRect(recPaint, recPaint.Left, recPaint.Top, Caption, FTextStyle);
end;

procedure TLEDStatus.CMMouseEnter(var Message: TLMMouse);
begin
  // only fire once
  if not FMouseInLEDStatus then
  begin
    FMouseInLEDStatus := true;
    Invalidate;
  end;
  inherited;
end;

procedure TLEDStatus.CMMouseLeave(var Message: TLMMouse);
begin
  // only fire once
  if FMouseInLEDStatus then
  begin
    FMouseInLEDStatus := false;
    Invalidate;
  end;
  inherited;
end;

//22.01.07-ud. Funktion WMMouseUp herausgenommen, dafr Click genommen. Das Event bleit im Nirvana hngen...
(*
procedure TLEDStatus.WMMouseUp(var Message: TLMMouseEvent);
begin
  If FMouseInLEDStatus then
  begin
    Case State of
      slGrayed :
        FLEDStatusState := slNolight;
      slNolight :
        FLEDStatusState := slLight;
      slLight :
        FLEDStatusState := slGrayed;
    end;
    Invalidate;
    //Click;
  end;
  //inherited;
end;
*)


procedure TLEDStatus.Click;
begin
  //22.01.07-ud. - BEGIN -
  If FMouseInLEDStatus then
  begin
    Case State of
      slGrayed :
        FLEDStatusState := slNolight;
      slNolight :
        FLEDStatusState := slLight;
      slLight :
        FLEDStatusState := slGrayed;
    end;
    Invalidate;
  end;
  //22.01.07-ud. - ENDE -
  inherited click;
end;

constructor TLEDStatus.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  controlstyle := controlstyle - [csAcceptsControls];
  With FTextStyle do
  begin
    Layout     := tlCenter;
    SingleLine := False;
    Clipping   := True;
    ExpandTabs := False;
    ShowPrefix := False;
    Opaque     := False;
  end;
  FColorLight   := clRed;  //22.01.07-ud. Muss hier noch einmal gesetzt werden, default in der
  FColorNolight := clGray; //22.01.07-ud. Property reicht nicht aus. (so auch in Delphi)
end;

destructor TLEDStatus.Destroy;
begin
  inherited Destroy;
end;

procedure TLEDStatus.Paint;
  function LEDRect(aRect : TRect) : TRect;
  var
    maxWidth : LongInt;
  begin
    // check the max-size
    if (aRect.Bottom - aRect.Top) <= (aRect.Right - aRect.Left) then
      maxWidth := (aRect.Bottom - aRect.Top)
    else
      maxWidth := (aRect.Right - aRect.Left);
    // look where the LED must be
    If Alignment = slRightJustify then
    begin
      // LED is right
      Result.Top := aRect.Top;
      Result.Bottom := aRect.Bottom;
      Result.Left := aRect.Right - maxWidth;
      Result.Right := aRect.Right;
    end
    else
    begin
      // LED is left
      Result.Top := aRect.Top;
      Result.Bottom := aRect.Bottom;
      Result.Left := aRect.Left;
      Result.Right := aRect.Left + maxWidth;
    end;
  end;
  
  function TextRect(aRect : TRect) : TRect;
  var
    maxWidth : LongInt;
  begin
    // check the max-size
    if (aRect.Bottom - aRect.Top) <= (aRect.Right - aRect.Left) then
      maxWidth := (aRect.Bottom - aRect.Top)
    else
      maxWidth := (aRect.Right - aRect.Left);
    // look where the LED must be
    If Alignment = slRightJustify then
    begin
      Result.Top := aRect.Top;
      Result.Bottom := aRect.Bottom;
      Result.Left := aRect.Left;
      Result.Right := aRect.Right - maxWidth;
    end
    else
    begin
      Result.Top := aRect.Top;
      Result.Bottom := aRect.Bottom;
      Result.Left := aRect.Left + maxWidth;
      Result.Right := aRect.Right;
    end;
  end;

var
  recPaint: TRect;
begin
  // clear the old
  // inherited;
  recPaint := Rect(0, 0, Width, Height);
  Canvas.Color := parent.Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.FillRect(ClientRect);
  //
  PaintLED(LEDRect(recPaint));
  PaintText(TextRect(recPaint));
end;


{ TLEDStatusRound }


procedure TLEDStatusRound.PaintLED(recPaint: TRect);
  // draw the LED for the component if needed
  Procedure DrawCircle(ValueColor : TColor; Rect : TRect; Style : TPenStyle);
  begin
    With Canvas, Rect do begin
      Pen.Style := Style;
      Pen.Color := ValueColor;
      Ellipse (Rect);
    end;
  end;

  Procedure FillCircle(ValueColor : TColor; Rect : TRect);
  begin
    With Canvas, Rect do begin
      InflateRect(Rect, -2, -2);
      Pen.Color := ValueColor;
      Brush.Color := ValueColor;
      Brush.Style := bsSolid;
      Pen.Style := psSolid;
      Ellipse (Rect);
    end;
  end;

var
 ARect : TRect;

begin
  Canvas.Brush.Color := parent.Color;
  Canvas.FillRect(recPaint);
  // Draw from Outer Circle to the inner Circle
  DrawCircle(clBlack, recPaint, psSolid);
  ARect := recPaint;
  InflateRect(ARect, -1, -1);
  DrawCircle(clGray, ARect, psSolid);
  if (not Enabled) then
    FillCircle(clBtnShadow, ARect)
  else
    case State of
      slGrayed : FillCircle(clBtnShadow, ARect);
      slNolight: FillCircle(FColorNolight, ARect);
      slLight  : FillCircle(FColorlight, ARect);
    end;
end;

{ TLEDStatusSquare }

procedure TLEDStatusSquare.PaintLED(recPaint: TRect);
var
 ARect : TRect;
begin
  // draw the LED for the component if needed
  Canvas.Brush.Color := parent.Color;
  Canvas.FillRect(recPaint);

  Canvas.Brush.Color := clBlack;
  Canvas.FillRect(recPaint);
  
  ARect := recPaint;
  InflateRect(ARect, -1, -1);

  Canvas.Brush.Color := clGray;
  Canvas.FillRect(ARect);

  InflateRect(ARect, -2, -2);

  if (not Enabled) then
  begin
    Canvas.Brush.Color := clBtnShadow;
    Canvas.FillRect(ARect);
  end
  else
  begin
    case State of
      slGrayed :
          begin
            Canvas.Brush.Color := clBtnShadow;
            Canvas.FillRect(ARect);
          end;
      slNolight:
          begin
            Canvas.Brush.Color := FColorNolight;
            Canvas.FillRect(ARect);
          end;
      slLight  :
          begin
            Canvas.Brush.Color := FColorlight;
            Canvas.FillRect(ARect);
          end;
    end;
  end;
end;


procedure Register;
begin
  RegisterComponents('AfBde',[TLEDStatusRound]);
  RegisterComponents('AfBde',[TLEDStatusSquare]);
end;

initialization

{$I led_status.lrs}


end.

